/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"PrintGeoAsX.h"

void
PrintGeoStuff (int print, int *faces, struct GEO Geo, int **pFaceList,
	       int **pEdgeList, int **pVerList)
{
  int i;

  (*pFaceList) = ut_alloc_1d_int (Geo.FaceQty + 1);
  (*pEdgeList) = ut_alloc_1d_int (Geo.EdgeQty + 1);
  (*pVerList) = ut_alloc_1d_int (Geo.VerQty + 1);

  if (print == 1)
  {
    PrintGeoFaceToPlot (Geo, faces, *pFaceList);
    PrintGeoEdgeToPlot (Geo, *pFaceList, *pEdgeList);
    PrintGeoVerToPlot (Geo, *pFaceList, *pVerList);
  }
  else if (print == 2)
  {
    for (i = 1; i <= Geo.EdgeQty; i++)
      if (Geo.EdgeState[i] == 0)
      {
	(*pEdgeList)[0]++;
	(*pEdgeList)[(*pEdgeList)[0]] = i;
      }
    for (i = 1; i <= Geo.VerQty; i++)
      if (Geo.VerState[i] >= 0)
      {
	(*pVerList)[0]++;
	(*pVerList)[(*pVerList)[0]] = i;
      }
  }

  return;
}

void
PrintGeoAsImg (struct GEO Geo, int *EdgeList,
	       int *VerList, char *format, char *filename)
{
  int i, j, edge, ver;
  FILE *out;

  out = OpenFile (filename, "w");

  if (strcmp (format, "asy") == 0)
  {
    fprintf (out, "import three;\n");
    fprintf (out, "currentprojection=obliqueX;\n");
    fprintf (out, "unitsize(10cm);\n");

    for (i = 1; i <= VerList[0]; i++)
    {
      ver = VerList[i];
      fprintf (out, "triple p%d =(%f, %f, %f);\n", VerList[i],
	       Geo.VerCoo[ver][0], Geo.VerCoo[ver][1], Geo.VerCoo[ver][2]);
    }
    for (i = 1; i <= EdgeList[0]; i++)
    {
      edge = EdgeList[i];
      fprintf (out, "draw (p%d--p%d);\n", Geo.EdgeVerNb[edge][0],
	       Geo.EdgeVerNb[edge][1]);
    }
  }
  else if (strcmp (format, "mast") == 0)
  {
    fprintf (out, "****master\n");
    fprintf (out, " ***geometry\n");
    for (i = 1; i <= VerList[0]; i++)
    {
      ver = VerList[i];
      fprintf (out, "  **point p%d\n", VerList[i]);
      fprintf (out, "   *position ");
      for (j = 0; j < 3; j++)
	fprintf (out, " %.12f", Geo.VerCoo[ver][j]);
      fprintf (out, "\n");
    }
    for (i = 1; i <= EdgeList[0]; i++)
    {
      edge = EdgeList[i];
      fprintf (out, "  **line l%d\n", edge);
      fprintf (out, "   *p1 p%d\n", Geo.EdgeVerNb[edge][0]);
      fprintf (out, "   *p2 p%d\n", Geo.EdgeVerNb[edge][1]);
    }

    fprintf (out, " ***plotting\n");
    fprintf (out, "  **default_options 1 1 1\n");
    fprintf (out, " ***lines3d\n");
    fprintf (out, "  **enable 0\n");
    fprintf (out, "****return\n");
  }

  CloseFile (out, filename, "w");

  return;
}
