/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"ReadMsh.h"

void
ReadThisNode (struct NODES *pNodes, int node, FILE * msh)
{
  int i;

  fscanf (msh, "%d", &(*pNodes).NodesNb[node]);
  for (i = 0; i <= 2; i++)
    fscanf (msh, "%lf", &(*pNodes).NodesCoo[node][i]);

  return;
}

void
ReadEltsProp (struct MESH *pMesh, int *EElsets,
	      int MaxEltQty, int *Residual, FILE * msh)
{
  int i, j, tmp, elttype, eltnodeqty;

  (*pMesh).EltQty = 0;

  if (MaxEltQty == 0)
  {
    neut_mesh_set_zero (pMesh);
    return;
  }

  if (Residual[1] == 0)
    fscanf (msh, "%d%d", &tmp, &elttype);
  else
  {
    tmp = Residual[0];
    elttype = Residual[1];
  }

  /* From the mesh dimension and elttype, we determine EltOrder.
   * if no solution is found, the data are recorded as Residual
   * and the routine returns.
   */
  (*pMesh).EltOrder = neut_elt_dim_gtype_order ((*pMesh).Dimension, elttype);

  if ((*pMesh).EltOrder == -1)
  {
    neut_mesh_set_zero (pMesh);
    Residual[0] = tmp;
    Residual[1] = elttype;
    return;
  }

  eltnodeqty = neut_elt_nodeqty ("tri", (*pMesh).Dimension, (*pMesh).EltOrder);

  (*pMesh).EltQty4Alloc = MaxEltQty;
  (*pMesh).EltNb = ut_alloc_1d_int (MaxEltQty + 1);
  (*pMesh).EltNodes = twoDIntDAlloc (MaxEltQty + 1, eltnodeqty);

  (*pMesh).EltQty++;
  (*pMesh).EltNb[1] = tmp;
  FTrash (1, msh);
  fscanf (msh, "%d", &EElsets[1]);
  FTrash (1, msh);

  for (j = 0; j < eltnodeqty; j++)
    fscanf (msh, "%d", &(*pMesh).EltNodes[1][j]);

  for (i = 2; i <= MaxEltQty; i++)
  {
    (*pMesh).EltQty++;

    fscanf (msh, "%d", &(*pMesh).EltNb[i]);

    fscanf (msh, "%d", &tmp);
    if (tmp != elttype)
    {
      Residual[0] = (*pMesh).EltNb[i];
      Residual[1] = tmp;
      (*pMesh).EltQty--;

      break;
    }

    FTrash (1, msh);
    fscanf (msh, "%d", &EElsets[i]);
    FTrash (1, msh);
    for (j = 0; j < eltnodeqty; j++)
      fscanf (msh, "%d", &(*pMesh).EltNodes[i][j]);
  }

  /* setting EltNbMax & EltNbInv */
  (*pMesh).EltNbMax = oneDIntMax ((*pMesh).EltNb, 1, (*pMesh).EltQty);
  (*pMesh).EltNbInv = ut_alloc_1d_int ((*pMesh).EltNbMax + 1);
  for (i = 1; i <= (*pMesh).EltQty; i++)
    (*pMesh).EltNbInv[(*pMesh).EltNb[i]] = i;

  return;
}

void
SetElsets (struct MESH *pMesh, int *EElsets)
{
  int i, qty, elsetnb, elsetnbinv;
  int *tmp;

  tmp = ut_alloc_1d_int ((*pMesh).EltQty + 1);
  ut_board_1d_int_memcpy (tmp + 1, (*pMesh).EltQty, EElsets + 1);

  /* sorting and compressing of tmp, to get the elset nbs. */
  tmp[0] = (*pMesh).EltQty;
  ut_board_1d_int_sortncompress (tmp + 1, tmp[0], &(tmp[0]));

  /* initialization of ElsetQty, ElsetNbMax, ElsetNb, ElsetNbInv */
  (*pMesh).ElsetQty = tmp[0];
  (*pMesh).ElsetNbMax = ut_board_1d_int_max (tmp + 1, tmp[0]);
  
  (*pMesh).ElsetNb = ut_alloc_1d_int (tmp[0] + 1);
  (*pMesh).ElsetNbInv = ut_alloc_1d_int ((*pMesh).ElsetNbMax + 1);
  for (i = 1; i <= tmp[0]; i++)
  {
    (*pMesh).ElsetNb[i] = tmp[i];
    (*pMesh).ElsetNbInv[(*pMesh).ElsetNb[i]] = i;
  }

  /* memory allocation of Elsets */
  (*pMesh).Elsets = oneDpIntDAlloc ((*pMesh).ElsetQty + 1);
  for (i = 1; i <= (*pMesh).ElsetQty; i++)
  {
    qty =
      oneDIntNbOfThisVal (EElsets, 1, (*pMesh).EltQty, (*pMesh).ElsetNb[i]);
    (*pMesh).Elsets[i] = ut_alloc_1d_int (qty + 1);
  }

  /* filling of Elsets */
  for (i = 1; i <= (*pMesh).EltQty; i++)
  {
    elsetnb = EElsets[i];
    elsetnbinv = (*pMesh).ElsetNbInv[elsetnb];
    (*pMesh).Elsets[elsetnbinv][0]++;
    (*pMesh).Elsets[elsetnbinv][(*pMesh).Elsets[elsetnbinv][0]] =
      (*pMesh).EltNb[i];
  }

  return;
}
