/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"InputData_fm.h"

void
TreatArg_fm (int fargc, char **fargv, int argc, char **argv,
	     struct IN *pIn, struct GEOPARA *pGeoPara)
{
  int inibody = 0;

  /* Setting default options */
  SetDefaultOptions_fm (pIn, pGeoPara);

  /* Reading options. */
  SetOptions_fm (pIn, pGeoPara, &((*pIn).body), &inibody, fargc, fargv);
  SetOptions_fm (pIn, pGeoPara, &((*pIn).body), &inibody, argc, argv);

  /* Testing options */
  if ((*pIn).input == -1)
  {
    ut_print_messagewnc (2, 72, "Bad input data.");
    abort ();
  }

  if ((*pIn).edgedis == 0 && (*pGeoPara).maxff > 1e-15)
  {
    ut_print_message (2,
		      "When using `-bwcy-edgedis 0', \"-maxff >0\" is disabled.\n");
    BadArguments ();
  }

  if ((*pIn).input == 1 &&
      ut_string_inlist ((*pIn).format, ',', "geof") &&
      (*pIn).liset > 0)
  {
    ut_print_message (2, "cannot output geof lisets with -restart geo.\n");
    abort ();
  }

  /* Writing file names */

  /* Recording body */
  if (inibody == 0)
  {
    (*pIn).body = ut_alloc_1d_char (1000);

    if ((*pIn).input == 0)
    {
      (*pIn).body = ut_alloc_1d_char (strlen ((*pIn).tess));
      (*pIn).body = ut_string_body ((*pIn).tess);
    }
    else /* input = 1 to 3 */
    {
      (*pIn).body = ut_alloc_1d_char (strlen ((*pIn).restart));
      (*pIn).body = ut_string_body ((*pIn).restart);
    }
  }

  /* Writing all file names */
  if ((*pIn).input != 0)
    (*pIn).tess = AddExtension ((*pIn).body, ".tess");

  if ((*pIn).input < 2)
    (*pIn).fod  = ut_string_changeextension ((*pIn).body, ".fod");
  else
    (*pIn).fod  = ut_string_changeextension ((*pIn).restart, ".fod");

  (*pIn).geo    = AddExtension ((*pIn).body, ".geo");
  (*pIn).msh    = AddExtension ((*pIn).body, ".msh");
  (*pIn).geof   = AddExtension ((*pIn).body, ".geof");
  (*pIn).geob   = AddExtension ((*pIn).body, ".geob");
  (*pIn).ff     = AddExtension ((*pIn).body, ".ff");
  (*pIn).edgel  = AddExtension ((*pIn).body, ".edgel");
  (*pIn).angle  = AddExtension ((*pIn).body, ".angle");
  (*pIn).msh    = AddExtension ((*pIn).body, ".msh");
  (*pIn).estat  = AddExtension ((*pIn).body, ".estat");
  (*pIn).eangle = AddExtension ((*pIn).body, ".eangle");
  (*pIn).nstat  = AddExtension ((*pIn).body, ".nstat");
  (*pIn).oin    = AddExtension ((*pIn).body, ".oin");
  (*pIn).mast   = AddExtension ((*pIn).body, ".mast");
  (*pIn).asy    = AddExtension ((*pIn).body, ".asy");
  if ((*pIn).maeextension == 0)
  {
    (*pIn).mae1 = AddExtension ((*pIn).body, ".mae1");
    (*pIn).mae2 = AddExtension ((*pIn).body, ".mae2");
    (*pIn).mae3 = AddExtension ((*pIn).body, ".mae3");
    (*pIn).mae4 = AddExtension ((*pIn).body, ".mae4");
    (*pIn).mae5 = AddExtension ((*pIn).body, ".mae5");
  }
  else
  {
    (*pIn).mae1 = AddExtension ((*pIn).body, ".parms");
    (*pIn).mae2 = AddExtension ((*pIn).body, ".mesh");
    (*pIn).mae3 = AddExtension ((*pIn).body, ".surf");
    (*pIn).mae4 = AddExtension ((*pIn).body, ".opt");
    (*pIn).mae5 = AddExtension ((*pIn).body, ".bcs");
  }

  if ((*pIn).input == 1)	/* checking if the input files exist. */
  {
    if (FileExists ((*pIn).geo) == 0)
      OpenFile ((*pIn).geo, "r");
    if (FileExists ((*pIn).fod) == 0)
      OpenFile ((*pIn).fod, "r");
  }

  return;
}

/* rcl2cl sets cl from rcl */
extern void
rcl2cl (struct TESS Tess, struct GEOPARA *pGeoPara)
{
  int i;
  double mindist;
  double **size = ut_alloc_2d (3, 2);

  (*pGeoPara).cl = (*pGeoPara).rcl * MeanEdgeLength (Tess);

  neut_tess_boundingbox (Tess, size);

  /* Calculating the minimum distance: */
  mindist = 1e32;
  for (i = 0; i <= 2; i++)
    mindist = DoubleMin (mindist, size[i][1] - size[i][0]);

  if ((*pGeoPara).cl > mindist)
  {
    LineHeader (1);
    printf ("Changing cl from %f to %f.\n", (*pGeoPara).cl, mindist);
    (*pGeoPara).cl = mindist;
  }

  twoDDoubleFree (size, 3);

  return;
}

double
MeanEdgeLength (struct TESS Tess)
{
  double **size = ut_alloc_2d (3, 2);
  double vol;

  neut_tess_boundingbox (Tess, size);

  /* Calculating the volume (assuming that the tessellation is a
   * parallelepiped).
   */
  vol =
    (size[0][1] - size[0][0]) * (size[1][1] - size[1][0]) * (size[2][1] -
							     size[2][0]);

  /* Formulae taken from Kumar & Kurtz, Materials characterization, 1995,
     and more accurately from Statistics of Three Dimensional Random Voronoi
     Tessellations, A. Brakke, March 2005:
     mean edge length (for a unit volume polyhedron) = 0.430857994283959
   */

  twoDDoubleFree (size, 3);

  return 0.430857994283959 / pow (Tess.PolyQty / vol, 0.3333333333333333);
}
