  /* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"InputData_fm.h"

/* SetDefaultOptions set the options to their default values */
void
SetDefaultOptions_fm (struct IN *pIn, struct GEOPARA *pGeoPara)
{
  /* General options --------------------------------------- */
  (*pIn).verbosity = 0;
  (*pIn).nset = 1;
  (*pIn).nsetlist = ut_alloc_1d_char (1000);
  (*pIn).gmsh = "gmsh";

  (*pIn).format = oneDCharDAlloc (100);
  sprintf ((*pIn).format, "geof");
  (*pIn).maeextension = 0;
  (*pIn).geo = NULL;
  (*pIn).msh = NULL;
  (*pIn).geof = NULL;
  (*pIn).mae1 = NULL;
  (*pIn).mae2 = NULL;
  (*pIn).mae3 = NULL;
  (*pIn).printfod = 1;
  (*pIn).fod = NULL;
  (*pIn).printestat = 0;
  (*pIn).estat = NULL;
  (*pIn).printnstat = 0;
  (*pIn).nstat = NULL;
  (*pIn).printedgel = 0;
  (*pIn).edgel = NULL;
  (*pIn).printangle = 0;
  (*pIn).angle = NULL;
  (*pIn).print = 0;
  (*pIn).printformat = ut_alloc_1d_char (100);
  sprintf ((*pIn).printformat, "asy");
  (*pIn).liset = 0;

  /* Options for the geometry modification ----------------- */
  (*pGeoPara).maxff = 0;
  (*pIn).newver = 1;
  (*pIn).mloop = 2;
  (*pIn).ffalgo = 1;
  (*pIn).edgedis = 1;
  (*pIn).geo2geob = 0;
  (*pIn).morder = 1;
  (*pIn).tess2geo = 1;
  (*pIn).renumber = 2;
  (*pIn).mesh = 1;
  (*pIn).tess = NULL;
  (*pIn).input = -1;
  (*pIn).eltfirstid = -1;	/* means default */
  (*pIn).nodefirstid = -1;	/* means default */

  /* Options for the meshing ------------------------------- */
  (*pGeoPara).cl = -1;		/* if unmodified directly, set from rcl. */
  (*pGeoPara).rcl = 1;
  (*pGeoPara).clmin = 0;
  (*pGeoPara).pcl = 2;
  (*pGeoPara).sel = -1;
  (*pGeoPara).rsel = -1;	/* relatively to the default value */
  (*pGeoPara).sa = 0;
  (*pGeoPara).sacl = -1;
  (*pGeoPara).sarcl = -1;
  (*pIn).meshalgorithm = 3;

  /* Other options ----------------------------------------- */
  /* Options for debugging --------------------------------- */
  (*pIn).meshimprovement2d = 1;
  (*pIn).delstop = 1e7;

  /* Useless or obsolete options ---------------------------
     (*pGeoPara).forcemergeqty = 0;
     (*pGeoPara).forbidmergeqty= 0;
   */

  (*pIn).printfaces = ut_alloc_1d_int (7);
  (*pIn).printfaces[0] = 3;
  (*pIn).printfaces[1] = 2;
  (*pIn).printfaces[2] = 4;
  (*pIn).printfaces[3] = 6;

  return;
}

void
SetOptions_fm (struct IN *pIn, struct GEOPARA *pGeoPara, char **pbody,
	       int *pinibody, int argc, char **argv)
{
  int i, j, ArgQty, Res;
  char **ArgList = twoDCharDAlloc (101, 101);
  char *Arg = oneDCharDAlloc (101);
  char *format = ut_alloc_1d_char (100);
  char* restart_type;

  /* This is the possible argument list. */
  ArgQty = 0;
  sprintf (ArgList[++ArgQty], "-o");
  sprintf (ArgList[++ArgQty], "-v");
  sprintf (ArgList[++ArgQty], "-gmsh");
  sprintf (ArgList[++ArgQty], "-maxff");
  sprintf (ArgList[++ArgQty], "-mloop");
  sprintf (ArgList[++ArgQty], "-cl");
  sprintf (ArgList[++ArgQty], "-rcl");
  sprintf (ArgList[++ArgQty], "-pcl");
  sprintf (ArgList[++ArgQty], "-clmin");
  sprintf (ArgList[++ArgQty], "-ffalgo");
  sprintf (ArgList[++ArgQty], "-morder");
  sprintf (ArgList[++ArgQty], "-edgel");
  sprintf (ArgList[++ArgQty], "-nset");
  sprintf (ArgList[++ArgQty], "-estat");
  sprintf (ArgList[++ArgQty], "-nstat");
  sprintf (ArgList[++ArgQty], "-geo2geob");
  sprintf (ArgList[++ArgQty], "-format");
  sprintf (ArgList[++ArgQty], "-sel");
  sprintf (ArgList[++ArgQty], "-rsel");
  sprintf (ArgList[++ArgQty], "-angle");
  sprintf (ArgList[++ArgQty], "-mesh");
  sprintf (ArgList[++ArgQty], "-convert");
  sprintf (ArgList[++ArgQty], "-meshalgorithm");
  sprintf (ArgList[++ArgQty], "-sa");
  sprintf (ArgList[++ArgQty], "-sacl");
  sprintf (ArgList[++ArgQty], "-sarcl");
  sprintf (ArgList[++ArgQty], "-print");
  sprintf (ArgList[++ArgQty], "-printformat");
  sprintf (ArgList[++ArgQty], "-printfaces");
  sprintf (ArgList[++ArgQty], "-liset");
  sprintf (ArgList[++ArgQty], "-nodefirstid");
  sprintf (ArgList[++ArgQty], "-eltfirstid");
  sprintf (ArgList[++ArgQty], "-maeextension");
  sprintf (ArgList[++ArgQty], "-restart");

  /* Reading arguments ----------------------------------- */
  for (i = 1; i < argc; i++)
  {
    /* Input file name */
    if (argv[i][0] != '-')
    {
      if ((*pIn).input == -1)
      {
	i--;
	(*pIn).tess = ut_arg_nextaschar (argv, &i, Arg);
	(*pIn).input = 0;
	(*pIn).tess2geo = 1;
      }
      else
	BadArguments ();
    }
    else
    {
      /* Searching option name (string completion stuff) */
      Res = StringComp (argv[i], ArgList, ArgQty, Arg);
      if (Res == 1)
      {
	LineHeader (2);
	printf ("Several possibilities for option `%s'.\n", argv[i]);
	BadArguments ();
      }
      else if (Res == -1)
      {
	LineHeader (2);
	printf ("Unknown option `%s'.\n", argv[i]);
	BadArguments ();
      }

      /* General options ------------------------------------------------ */
      if (strcmp (Arg, "-o") == 0 && i < argc - 1)
      {
	(*pbody) = NextArgAsChar (argv, &i, Arg);
	(*pbody) = GiveBody (*pbody);
	(*pinibody) = 1;
      }
      else if (strcmp (Arg, "-v") == 0 && i < argc - 1)
	(*pIn).verbosity = NextArgAsInt (argv, &i, Arg, 0, 1);

      else if (strcmp (Arg, "-gmsh") == 0 && i < argc - 1)
	(*pIn).gmsh = NextArgAsChar (argv, &i, Arg);


      /* Options for the geometry modification ---------------------------- */
      else if (strcmp (Arg, "-maxff") == 0 && i < argc - 1)
	(*pGeoPara).maxff = NextArgAsReal (argv, &i, Arg, 0., 90.);
      else if (strcmp (Arg, "-mloop") == 0 && i < argc - 1)
	(*pIn).mloop = NextArgAsInt (argv, &i, Arg, 0, 1e7);

      /* Options for the meshing ----------------------------------------- */

      else if (strcmp (Arg, "-cl") == 0 && i < argc - 1)
	(*pGeoPara).cl = NextArgAsReal (argv, &i, Arg, 0., 1.e30);
      else if (strcmp (Arg, "-rcl") == 0 && i < argc - 1)
	(*pGeoPara).rcl = NextArgAsReal (argv, &i, Arg, 0., 1.e30);
      else if (strcmp (Arg, "-sa") == 0 && i < argc - 1)
	(*pGeoPara).sa = NextArgAsReal (argv, &i, Arg, 0., 180.);
      else if (strcmp (Arg, "-sacl") == 0 && i < argc - 1)
	(*pGeoPara).sacl = NextArgAsReal (argv, &i, Arg, 0., 1.e30);
      else if (strcmp (Arg, "-sarcl") == 0 && i < argc - 1)
	(*pGeoPara).sarcl = NextArgAsReal (argv, &i, Arg, 0., 1.e30);
      else if (strcmp (Arg, "-clmin") == 0 && i < argc - 1)
	(*pGeoPara).clmin = NextArgAsReal (argv, &i, Arg, 0., 1.e30);
      else if (strcmp (Arg, "-pcl") == 0 && i < argc - 1)
	(*pGeoPara).pcl = NextArgAsReal (argv, &i, Arg, 1., 1.e30);
      else if (strcmp (Arg, "-sel") == 0 && i < argc - 1)
	(*pGeoPara).sel = NextArgAsReal (argv, &i, Arg, 0., 1.e30);
      else if (strcmp (Arg, "-rsel") == 0 && i < argc - 1)
	(*pGeoPara).rsel = NextArgAsReal (argv, &i, Arg, 0., 1.e30);
      else if (strcmp (Arg, "-meshalgorithm") == 0 && i < argc - 1)
	(*pIn).meshalgorithm = NextArgAsInt (argv, &i, Arg, 1, 3);
      else if (strcmp (Arg, "-ffalgo") == 0 && i < argc - 1)
	(*pIn).ffalgo = NextArgAsInt (argv, &i, Arg, 0, 2);
      else if (strcmp (Arg, "-morder") == 0 && i < argc - 1)
	(*pIn).morder = NextArgAsInt (argv, &i, Arg, 1, 2);
      else if (strcmp (Arg, "-edgel") == 0 && i < argc - 1)
	(*pIn).printedgel = NextArgAsInt (argv, &i, Arg, 0, 1);
      else if (strcmp (Arg, "-angle") == 0 && i < argc - 1)
	(*pIn).printangle = NextArgAsInt (argv, &i, Arg, 0, 1);
      else if (strcmp (Arg, "-nset") == 0 && i < argc - 1)
      {
	(*pIn).nset = NextArgAsInt (argv, &i, Arg, 0, 4);
	if ((*pIn).nset == 4)
	  (*pIn).nsetlist = NextArgAsChar (argv, &i, Arg);
      }
      else if (strcmp (Arg, "-estat") == 0 && i < argc - 1)
	(*pIn).printestat = NextArgAsInt (argv, &i, Arg, 0, 1);
      else if (strcmp (Arg, "-nstat") == 0 && i < argc - 1)
	(*pIn).printnstat = NextArgAsInt (argv, &i, Arg, 0, 1);
      else if (strcmp (Arg, "-geo2geob") == 0 && i < argc - 1)
	(*pIn).geo2geob = NextArgAsInt (argv, &i, Arg, 0, 1);
      else if (strcmp (Arg, "-nodefirstid") == 0 && i < argc - 1)
	(*pIn).nodefirstid = NextArgAsInt (argv, &i, Arg, 0, 1e7);
      else if (strcmp (Arg, "-eltfirstid") == 0 && i < argc - 1)
	(*pIn).eltfirstid = NextArgAsInt (argv, &i, Arg, 0, 1e7);
      else if (strcmp (Arg, "-liset") == 0 && i < argc - 1)
	(*pIn).liset = NextArgAsInt (argv, &i, Arg, 0, 1);
      else if (strcmp (Arg, "-print") == 0 && i < argc - 1)
	(*pIn).print = NextArgAsInt (argv, &i, Arg, 0, 2);
      else if (strcmp (Arg, "-printformat") == 0 && i < argc - 1)
      {
	(*pIn).printformat = ut_alloc_1d_char (100);
	(*pIn).printformat = NextArgAsChar (argv, &i, Arg);
      }
      else if (strcmp (Arg, "-printfaces") == 0 && i < argc - 1)
      {
	(*pIn).printfaces[0] = NextArgAsInt (argv, &i, Arg, 1, 6);
	for (j = 1; j <= (*pIn).printfaces[0]; j++)
	  (*pIn).printfaces[j] = NextArgAsInt (argv, &i, Arg, 1, 6);
      }

      else if (strcmp (Arg, "-maeextension") == 0 && i < argc - 1)
	(*pIn).maeextension = NextArgAsInt (argv, &i, Arg, 0, 1);
      else if (strcmp (Arg, "-restart") == 0 && i < argc - 2)
      {
	restart_type = ut_alloc_1d_char (strlen (argv[i + 1]));
	restart_type = ut_arg_nextaschar (argv, &i, Arg);
	if (strcmp (restart_type, "geo") == 0)
	{
	  (*pIn).input = 1;
	  (*pIn).tess2geo = 0;
	}
	else if (strcmp (restart_type, "msh") == 0)
	{
	  (*pIn).input = 2;
	  (*pIn).tess2geo = 0;
	  (*pIn).mesh = 0;
	}
	else if (strcmp (restart_type, "geof") == 0)
	{
	  (*pIn).input = 3;
	  (*pIn).tess2geo = 0;
	  (*pIn).mesh = 0;
	}
	else if (strcmp (restart_type, "mae") == 0)
	{
	  (*pIn).input = 4;
	  (*pIn).tess2geo = 0;
	  (*pIn).mesh = 0;
	}

	ut_free_1d_char (restart_type);

	(*pIn).restart = ut_alloc_1d_char (strlen (argv[i + 1]));
	(*pIn).restart = ut_arg_nextaschar (argv, &i, Arg);
      }
      else if (strcmp (Arg, "-format") == 0 && i < argc - 1)
      {
	(*pIn).format = NextArgAsChar (argv, &i, Arg);
	if (ut_string_inlist ((*pIn).format, ',', "msh") == 0 &&
	    ut_string_inlist ((*pIn).format, ',', "geof") == 0 &&
	    ut_string_inlist ((*pIn).format, ',', "mae") == 0)
	  (*pIn).mesh = 0;
      }
      else
      {
	if (i == 1)
	  BadArguments ();
	else
	  ArgError (Arg);
      }
    }
  }

  twoDCharFree (ArgList, 101);
  oneDCharFree (Arg);
  ut_free_1d_char (format);

  return;
}
