/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"MeshRenumber.h"

/* level=1: renumbering to get continuous sequences (1,2,3,...)
 * level=2: crude renumbering according to a cubic array to get
 * successive nodes & elements not too far from the one the the other.
 */
void
MeshRenumber (int level, int partqty, struct NODES *pNodes, struct MESH *pMesh2D,
	      struct MESH *pMesh3D, int eltfirstid, int nodefirstid)
{
  int *nodes_old_new = ut_alloc_1d_int ((*pNodes).NodeNbMax + 1);
  int *elts2d_old_new = ut_alloc_1d_int ((*pMesh2D).EltNbMax + 1);
  int *elts3d_old_new = ut_alloc_1d_int ((*pMesh3D).EltNbMax + 1);

  if (level == 1)
  {
    neut_nodes_renumber_continuous (pNodes, nodefirstid, nodes_old_new);
    neut_mesh_renumber_continuous (pMesh2D, eltfirstid, nodes_old_new, elts2d_old_new);
    neut_mesh_renumber_continuous (pMesh3D, eltfirstid, nodes_old_new, elts3d_old_new);
  }
  else if (level == 2)
  {
    neut_nodes_renumber_continuous (pNodes, 1, nodes_old_new);
    neut_mesh_renumber_continuous (pMesh2D, 1, nodes_old_new, elts2d_old_new);
    neut_mesh_renumber_continuous (pMesh3D, 1, nodes_old_new, elts3d_old_new);
    neut_mesh_renumber_crude (pNodes, pMesh2D, pMesh3D);
    neut_nodes_renumber_continuous (pNodes, nodefirstid, nodes_old_new);
    neut_mesh_renumber_continuous (pMesh2D, eltfirstid, nodes_old_new, elts2d_old_new);
    neut_mesh_renumber_continuous (pMesh3D, eltfirstid, nodes_old_new, elts3d_old_new);
  }


  ut_free_1d_int (nodes_old_new);
  ut_free_1d_int (elts2d_old_new);
  ut_free_1d_int (elts3d_old_new);
  
  return;
}
