/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"WriteMeshGeof.h"

void
neut_mesh_fprintf_geof_head (FILE* file)
{
  fprintf (file, "***geometry\n");

  return;
}

void
neut_mesh_fprintf_geof_nodes (FILE* out, struct NODES Nodes)
{
  int i, j;

  fprintf (out, "**node\n");
  fprintf (out, "%d 3\n", Nodes.NodeQty);
  for (i = 1; i <= Nodes.NodeQty; i++)
  {
    fprintf (out, "%d", Nodes.NodesNb[i]);
    for (j = 0; j < 3; j++)
      fprintf (out, "  %.12f", Nodes.NodesCoo[i][j]);
    fprintf (out, "\n");
  }

  return;
}

void
neut_mesh_fprintf_geof_elts (FILE* out, struct MESH Mesh)
{
  int i, j;
  char **fodnames = ut_alloc_2d_char (7, 3);
  sprintf (fodnames[1], "x0");
  sprintf (fodnames[2], "x1");
  sprintf (fodnames[3], "y0");
  sprintf (fodnames[4], "y1");
  sprintf (fodnames[5], "z0");
  sprintf (fodnames[6], "z1");

  fprintf (out, "**element\n");
  fprintf (out, "%d\n", Mesh.EltQty);

  if (Mesh.Dimension == 3 && Mesh.EltOrder == 1)
    for (i = 1; i <= Mesh.EltQty; i++)
    {
      fprintf (out, "%d c3d4", Mesh.EltNb[i]);
      for (j = 0; j < 4; j++)
	fprintf (out, " %d", Mesh.EltNodes[i][j]);
      fprintf (out, "\n");
    }
  else if (Mesh.Dimension == 3 && Mesh.EltOrder == 2)
    for (i = 1; i <= Mesh.EltQty; i++)
    {
      fprintf (out, "%d c3d10", Mesh.EltNb[i]);
      fprintf (out, " %d", Mesh.EltNodes[i][0]);
      fprintf (out, " %d", Mesh.EltNodes[i][1]);
      fprintf (out, " %d", Mesh.EltNodes[i][2]);
      fprintf (out, " %d", Mesh.EltNodes[i][4]);
      fprintf (out, " %d", Mesh.EltNodes[i][5]);
      fprintf (out, " %d", Mesh.EltNodes[i][6]);
      fprintf (out, " %d", Mesh.EltNodes[i][7]);
      fprintf (out, " %d", Mesh.EltNodes[i][9]);
      fprintf (out, " %d", Mesh.EltNodes[i][8]);
      fprintf (out, " %d", Mesh.EltNodes[i][3]);
      fprintf (out, "\n");
    }

  return;
}

void
neut_mesh_fprintf_geof_nsets (FILE* file, int nset, char* nsetlist,
   int** FoDNodes, int** FoDBNodes, int** EoDNodes,
   int** EoDBNodes, int** CoDNodes)
{
  int i;
  char NsetNames[100][100];

  if (nset >= 1)
  {
    sprintf (NsetNames[1], "x0");
    sprintf (NsetNames[2], "x1");
    sprintf (NsetNames[3], "y0");
    sprintf (NsetNames[4], "y1");
    sprintf (NsetNames[5], "z0");
    sprintf (NsetNames[6], "z1");

    for (i = 1; i <= 6; i++)
      if (nset < 4
	  || (nset == 4
	      && ut_string_inlist (nsetlist, ',', NsetNames[i])) == 1)
	neut_mesh_fprintf_geof_nset (file, NsetNames[i], FoDNodes[i]);
  }

  if (nset >= 2)
  {
    sprintf (NsetNames[1], "x0y0z0");
    sprintf (NsetNames[2], "x1y0z0");
    sprintf (NsetNames[3], "x0y1z0");
    sprintf (NsetNames[4], "x1y1z0");
    sprintf (NsetNames[5], "x0y0z1");
    sprintf (NsetNames[6], "x1y0z1");
    sprintf (NsetNames[7], "x0y1z1");
    sprintf (NsetNames[8], "x1y1z1");

    for (i = 1; i <= 8; i++)
      if (nset < 4
	  || (nset == 4
	      && ut_string_inlist (nsetlist, ',', NsetNames[i])) == 1)
	neut_mesh_fprintf_geof_nset (file, NsetNames[i], CoDNodes[i]);

    sprintf (NsetNames[1], "y0z0");
    sprintf (NsetNames[2], "y1z0");
    sprintf (NsetNames[3], "y0z1");
    sprintf (NsetNames[4], "y1z1");
    sprintf (NsetNames[5], "x0z0");
    sprintf (NsetNames[6], "x0z1");
    sprintf (NsetNames[7], "x1z0");
    sprintf (NsetNames[8], "x1z1");
    sprintf (NsetNames[9], "x0y0");
    sprintf (NsetNames[10], "x1y0");
    sprintf (NsetNames[11], "x0y1");
    sprintf (NsetNames[12], "x1y1");

    for (i = 1; i <= 12; i++)
      if (nset < 4
	  || (nset == 4
	      && ut_string_inlist (nsetlist, ',', NsetNames[i])) == 1)
	neut_mesh_fprintf_geof_nset (file, NsetNames[i], EoDNodes[i]);
  }

  if (nset >= 3)
  {
    sprintf (NsetNames[1], "x0body");
    sprintf (NsetNames[2], "x1body");
    sprintf (NsetNames[3], "y0body");
    sprintf (NsetNames[4], "y1body");
    sprintf (NsetNames[5], "z0body");
    sprintf (NsetNames[6], "z1body");

    for (i = 1; i <= 6; i++)
      if (nset < 4
	  || (nset == 4
	      && ut_string_inlist (nsetlist, ',', NsetNames[i])) == 1)
	neut_mesh_fprintf_geof_nset (file, NsetNames[i], FoDBNodes[i]);

    sprintf (NsetNames[1], "y0z0body");
    sprintf (NsetNames[2], "y1z0body");
    sprintf (NsetNames[3], "y0z1body");
    sprintf (NsetNames[4], "y1z1body");
    sprintf (NsetNames[5], "x0z0body");
    sprintf (NsetNames[6], "x0z1body");
    sprintf (NsetNames[7], "x1z0body");
    sprintf (NsetNames[8], "x1z1body");
    sprintf (NsetNames[9], "x0y0body");
    sprintf (NsetNames[10], "x1y0body");
    sprintf (NsetNames[11], "x0y1body");
    sprintf (NsetNames[12], "x1y1body");

    for (i = 1; i <= 12; i++)
      if (nset < 4
	  || (nset == 4
	      && ut_string_inlist (nsetlist, ',', NsetNames[i])) == 1)
	neut_mesh_fprintf_geof_nset (file, NsetNames[i], EoDBNodes[i]);
  }

}

void
neut_mesh_fprintf_geof_sethead (FILE* file)
{
  fprintf (file, "***group\n");

  return;
}

void
neut_mesh_fprintf_geof_lisets (struct GEO Geo, int **FoD, struct NODES Nodes,
	       struct MESH Mesh2D, FILE * out)
{
  int i, j, k, qty, edge, face, idinv, idinv2;
  int face1, face2, face1nodeqty, face2nodeqty;
  int *edge_list;
  int *face1nodes;
  int *face2nodes;
  int *node_list;
  int nodeqty;
  char **LisetNames = ut_alloc_2d_char (7, 100);
  double *ref = ut_alloc_1d (3);
  double *uvect = ut_alloc_1d (3);
  double *vect = ut_alloc_1d (3);
  double *coo;
  int *index;

  sprintf (LisetNames[1], "x0");
  sprintf (LisetNames[2], "x1");
  sprintf (LisetNames[3], "y0");
  sprintf (LisetNames[4], "y1");
  sprintf (LisetNames[5], "z0");
  sprintf (LisetNames[6], "z1");

  for (i = 1; i <= 6; i++)	/* for every face of the domain */
  {
    fprintf (out, "**liset %s\n", LisetNames[i]);

    /* all edges of the faces must be plotted; recording them */
    qty = 0;
    for (j = 1; j <= FoD[i][0]; j++)
      qty += Geo.FaceVerQty[FoD[i][j]];

    edge_list = ut_alloc_1d_int (qty);

    for (j = 1; j <= FoD[i][0]; j++)
    {
      face = FoD[i][j];

      for (k = 1; k <= Geo.FaceVerQty[face]; k++)
      {
	edge = Geo.FaceEdgeNb[face][k];

	if (ut_board_1d_int_eltpos (edge_list + 1, edge_list[0], edge) == -1)
	  edge_list[++edge_list[0]] = edge;
      }
    }

    /* for every edge, plotting liset */
    for (j = 1; j <= edge_list[0]; j++)
    {
      edge = edge_list[j];
      face1 = Geo.EdgeFaceNb[edge][0];
      face2 = Geo.EdgeFaceNb[edge][1];

      /* building the list of nodes */
      neut_mesh_elsetnodes (Mesh2D, face1, &face1nodes, &face1nodeqty);
      neut_mesh_elsetnodes (Mesh2D, face2, &face2nodes, &face2nodeqty);

      ut_board_1d_inter (face1nodes, face1nodeqty,
			 face2nodes, face2nodeqty, &node_list, &nodeqty);
      ut_free_1d_int (face1nodes);
      ut_free_1d_int (face2nodes);

      idinv = Nodes.NodesNbInv[node_list[0]];
      ut_board_1d_memcpy (ref, 3, Nodes.NodesCoo[idinv]);
      idinv2 = Nodes.NodesNbInv[node_list[1]];
      ut_board_1d_sub (ref, Nodes.NodesCoo[idinv2], 3, uvect);

      coo = ut_alloc_1d (nodeqty);
      for (k = 0; k < nodeqty; k++)
      {
	idinv2 = Nodes.NodesNbInv[node_list[k]];
	ut_board_1d_sub (ref, Nodes.NodesCoo[idinv2], 3, vect);
	coo[k] = ut_vector_scalprod (uvect, vect);
      }

      index = ut_alloc_1d_int (nodeqty);
      ut_board_1d_sort_index (coo, nodeqty, index);
      ut_board_1d_permutation_int (node_list, nodeqty, index);

      if (Mesh2D.EltOrder == 1)
	for (k = 0; k < nodeqty - 1; k++)
	  fprintf (out, "line %d %d\n", node_list[k], node_list[k + 1]);
      else if (Mesh2D.EltOrder == 2)
	for (k = 0; k < nodeqty - 2; k += 2)
	  fprintf (out, "quad %d %d %d\n", node_list[k], node_list[k + 1],
		   node_list[k + 2]);
      else
	abort ();

      ut_free_1d (coo);
      ut_free_1d_int (index);
      ut_free_1d_int (node_list);
    }

    ut_free_1d_int (edge_list);
  }

  ut_free_2d_char (LisetNames, 7);
  ut_free_1d (uvect);
  ut_free_1d (vect);

  return;
}

void
neut_mesh_fprintf_geof_elsets (FILE* file, struct MESH Mesh3D)
{
  int i, j, col;

  for (i = 1; i <= Mesh3D.ElsetQty; i++)
  {
    fprintf (file, "**elset poly%d\n", i);
    col = 0;
    for (j = 1; j <= Mesh3D.Elsets[i][0]; j++)
      FPrintfWNC (file, Mesh3D.Elsets[i][j], &col, 72);

    fprintf (file, "\n");
  }

  return;
}

void
neut_mesh_fprintf_geof_foot (FILE * out)
{
  fprintf (out, "***return\n");

  return;
}
