/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"WriteMeshMae.h"

void
neut_mesh_fprintf_mae_parms (FILE* file, struct NODES Nodes, struct MESH Mesh)
{
  /* mae1 = nb of elts and nodes */
  fprintf (file, "%d %d 9\n", Mesh.EltQty, Nodes.NodeQty);
}

void
neut_mesh_fprintf_mae_nodes (FILE * file, struct NODES Nodes)
{
  int i, j;

  for (i = 1; i <= Nodes.NodeQty; i++)
  {
    fprintf (file, "%d", Nodes.NodesNb[i]);
    for (j = 0; j < 3; j++)
      fprintf (file, "  %.12f", Nodes.NodesCoo[i][j]);
    fprintf (file, "\n");
  }

  return;
}

void
neut_mesh_fprintf_mae_elts (FILE * file, struct MESH Mesh, int **FoD)
{
  int i, j, eltnb, elsetnb, fod;
  char **fodnames = ut_alloc_2d_char (7, 3);
  sprintf (fodnames[1], "x0");
  sprintf (fodnames[2], "x1");
  sprintf (fodnames[3], "y0");
  sprintf (fodnames[4], "y1");
  sprintf (fodnames[5], "z0");
  sprintf (fodnames[6], "z1");

  if (Mesh.Dimension == 2 && Mesh.EltOrder == 1)
  {
    fprintf (file, "1\n      %d\n", Mesh.EltQty);
    for (i = 1; i <= Mesh.EltQty; i++)
    {
      eltnb = Mesh.EltNb[i];
      fprintf (file, "%d ", eltnb);
      for (j = 0; j < 3; j++)
	fprintf (file, " %d", Mesh.EltNodes[i][j]);

      /* searching fod via elset */
      if (neut_mesh_eltelset (Mesh, eltnb, &elsetnb) < 0)
	abort ();
      if (neut_fod_facefod (FoD, elsetnb, &fod) < 0)
	abort ();

      fprintf (file, " %s\n", fodnames[fod]);
    }
  }
  else if (Mesh.Dimension == 2 && Mesh.EltOrder == 2)
  {
    fprintf (file, "1\n%8d\n", Mesh.EltQty);
    for (i = 1; i <= Mesh.EltQty; i++)
    {
      eltnb = Mesh.EltNb[i];
      fprintf (file, "%d ", eltnb);
      fprintf (file, " %d", Mesh.EltNodes[i][0]);
      fprintf (file, " %d", Mesh.EltNodes[i][3]);
      fprintf (file, " %d", Mesh.EltNodes[i][1]);
      fprintf (file, " %d", Mesh.EltNodes[i][4]);
      fprintf (file, " %d", Mesh.EltNodes[i][2]);
      fprintf (file, " %d", Mesh.EltNodes[i][5]);

      /* searching fod via elset */
      if (neut_mesh_eltelset (Mesh, eltnb, &elsetnb) < 0)
	abort ();
      if (neut_fod_facefod (FoD, elsetnb, &fod) < 0)
	abort ();

      fprintf (file, " %s\n", fodnames[fod]);
    }
  }
  else if (Mesh.Dimension == 3 && Mesh.EltOrder == 1)
    for (i = 1; i <= Mesh.EltQty; i++)
    {
      fprintf (file, "%d ", Mesh.EltNb[i]);
      for (j = 0; j < 4; j++)
	fprintf (file, " %d", Mesh.EltNodes[i][j]);
      fprintf (file, "\n");
    }
  else if (Mesh.Dimension == 3 && Mesh.EltOrder == 2)
    for (i = 1; i <= Mesh.EltQty; i++)
    {
      fprintf (file, "%d ", Mesh.EltNb[i]);
      fprintf (file, " %d", Mesh.EltNodes[i][0]);
      fprintf (file, " %d", Mesh.EltNodes[i][4]);
      fprintf (file, " %d", Mesh.EltNodes[i][1]);
      fprintf (file, " %d", Mesh.EltNodes[i][5]);
      fprintf (file, " %d", Mesh.EltNodes[i][2]);
      fprintf (file, " %d", Mesh.EltNodes[i][6]);
      fprintf (file, " %d", Mesh.EltNodes[i][7]);
      fprintf (file, " %d", Mesh.EltNodes[i][9]);
      fprintf (file, " %d", Mesh.EltNodes[i][8]);
      fprintf (file, " %d", Mesh.EltNodes[i][3]);
      fprintf (file, "\n");
    }

  return;
}

/* coded for mae only, where, moreover, the elt nbs have to be changed for those of the 3D underlying elts */
void
neut_mesh_fprintf_mae_skinelts (FILE * file, struct MESH Mesh2D,
				 struct MESH Mesh3D, struct NODES Nodes,
				 int **FoD)
{
  double res;
  int i, j, k, l, eltnb, eltnbinv, elsetnbinv, eltqty, elt3d;
  char **fodnames = ut_alloc_2d_char (7, 2);
  double *n = ut_alloc_1d (3);
  double *n0 = ut_alloc_1d (3);
  int seqo1[3] = { 0, 1, 2 };
  int seqo2[6] = { 0, 3, 1, 4, 2, 5 };

  sprintf (fodnames[1], "x0");
  sprintf (fodnames[2], "x1");
  sprintf (fodnames[3], "y0");
  sprintf (fodnames[4], "y1");
  sprintf (fodnames[5], "z0");
  sprintf (fodnames[6], "z1");

  if (Mesh2D.Dimension != 2)
  {
    ut_print_messagewnc (1, 72, "2D skin mesh not available. surf file cannot be written.");
    return;
  }
  
    eltqty = 0;
    for (i = 1; i <= 6; i++)
      for (j = 1; j <= FoD[i][0]; j++)
      {
	elsetnbinv = Mesh2D.ElsetNbInv[FoD[i][j]];
	eltqty += Mesh2D.Elsets[elsetnbinv][0];
      }

    fprintf (file, "1\n%d\n", eltqty);

    for (i = 1; i <= 6; i++)
    {
      ut_board_1d_set (n0, 3, 0);
      if (i == 1)
	n0[0] = -1;
      else if (i == 2)
	n0[0] = 1;
      else if (i == 3)
	n0[1] = -1;
      else if (i == 4)
	n0[1] = 1;
      else if (i == 5)
	n0[2] = -1;
      else if (i == 6)
	n0[2] = 1;

      for (j = 1; j <= FoD[i][0]; j++)
      {
	elsetnbinv = Mesh2D.ElsetNbInv[FoD[i][j]];

	for (k = 1; k <= Mesh2D.Elsets[elsetnbinv][0]; k++)
	{
	  eltnb = Mesh2D.Elsets[elsetnbinv][k];
	  eltnbinv = Mesh2D.EltNbInv[eltnb];
	  neut_mesh_eltnormal (Mesh2D, Nodes, eltnb, n);

	  if (neut_mesh_elt2delt3d (Mesh2D, eltnb, Mesh3D, &elt3d) < 0)
	    abort ();

	  fprintf (file, "%d", elt3d);

	  res = ut_vector_scalprod (n0, n);
	  if (Mesh2D.EltOrder == 1)
	  {
	    if (res > 0)
	      for (l = 0; l <= 2; l++)
		fprintf (file, " %d", Mesh2D.EltNodes[eltnbinv][seqo1[l]]);
	    else
	      for (l = 2; l >= 0; l--)
		fprintf (file, " %d", Mesh2D.EltNodes[eltnbinv][seqo1[l]]);
	  }
	  else if (Mesh2D.EltOrder == 2)
	  {
	    if (res > 0)
	    {
	      for (l = 0; l <= 5; l++)
		fprintf (file, " %d", Mesh2D.EltNodes[eltnbinv][seqo2[l]]);
	    }
	    else
	    {
	      /* reverse order from 0 (must be from a 1st order node) */
	      fprintf (file, " %d", Mesh2D.EltNodes[eltnbinv][seqo2[0]]);
	      for (l = 5; l >= 1; l--)
		fprintf (file, " %d", Mesh2D.EltNodes[eltnbinv][seqo2[l]]);
	    }
	  }

	  fprintf (file, " %s\n", fodnames[i]);
	}
      }
    }

  ut_free_1d (n);
  ut_free_1d (n0);
  ut_free_2d_char (fodnames, 7);

  return;
}

void
neut_mesh_fprintf_mae_elsets (FILE * file, struct MESH Mesh3D)
{
  int i, elset;

  fprintf (file, "grain-input\n");
  fprintf (file, "%d %d\n", Mesh3D.EltQty, Mesh3D.ElsetQty);
  for (i = 1; i <= Mesh3D.EltQty; i++)
  {
    elset = neut_mesh_elt_elset (Mesh3D, Mesh3D.EltNb[i]);
    fprintf (file, "%d 1\n", elset);	/* 1 if for the phase */
  }
  fprintf (file, "end-options\n");

  return;
}

void
neut_mesh_fprintf_mae_nsets (FILE * file, int nset, char *nsetlist,
			     int **FoDNodes, int **FoDBNodes, int **EoDNodes,
			     int **EoDBNodes, int **CoDNodes)
{
  int i, nodeqty;
  char NsetNames[100][100];

  fprintf (file, "0\n");

  /* printing node qty */

  nodeqty = 0;
  for (i = 1; i <= 6; i++)
    nodeqty += FoDNodes[i][0];

  if (nset >= 2)
  {
    nodeqty += 8;
    for (i = 1; i <= 12; i++)
      nodeqty += EoDNodes[i][0];
  }

  if (nset >= 3)
  {
    for (i = 1; i <= 6; i++)
      nodeqty += FoDBNodes[i][0];
    for (i = 1; i <= 12; i++)
      nodeqty += EoDBNodes[i][0];
  }

  fprintf (file, "%d\n", nodeqty);

  /* printing nsets */

  if (nset >= 1)
  {
    sprintf (NsetNames[1], "x0");
    sprintf (NsetNames[2], "x1");
    sprintf (NsetNames[3], "y0");
    sprintf (NsetNames[4], "y1");
    sprintf (NsetNames[5], "z0");
    sprintf (NsetNames[6], "z1");

    for (i = 1; i <= 6; i++)
      if (nset < 4
	  || (nset == 4
	      && ut_string_inlist (nsetlist, ',', NsetNames[i])) == 1)
	neut_mesh_fprintf_mae_nset (file, NsetNames[i], FoDNodes[i]);
  }

  if (nset >= 2)
  {
    sprintf (NsetNames[1], "x0y0z0");
    sprintf (NsetNames[2], "x1y0z0");
    sprintf (NsetNames[3], "x0y1z0");
    sprintf (NsetNames[4], "x1y1z0");
    sprintf (NsetNames[5], "x0y0z1");
    sprintf (NsetNames[6], "x1y0z1");
    sprintf (NsetNames[7], "x0y1z1");
    sprintf (NsetNames[8], "x1y1z1");

    for (i = 1; i <= 8; i++)
      if (nset < 4
	  || (nset == 4
	      && ut_string_inlist (nsetlist, ',', NsetNames[i])) == 1)
	neut_mesh_fprintf_mae_nset (file, NsetNames[i], CoDNodes[i]);

    sprintf (NsetNames[1], "y0z0");
    sprintf (NsetNames[2], "y1z0");
    sprintf (NsetNames[3], "y0z1");
    sprintf (NsetNames[4], "y1z1");
    sprintf (NsetNames[5], "x0z0");
    sprintf (NsetNames[6], "x0z1");
    sprintf (NsetNames[7], "x1z0");
    sprintf (NsetNames[8], "x1z1");
    sprintf (NsetNames[9], "x0y0");
    sprintf (NsetNames[10], "x1y0");
    sprintf (NsetNames[11], "x0y1");
    sprintf (NsetNames[12], "x1y1");

    for (i = 1; i <= 12; i++)
      if (nset < 4
	  || (nset == 4
	      && ut_string_inlist (nsetlist, ',', NsetNames[i])) == 1)
	neut_mesh_fprintf_mae_nset (file, NsetNames[i], EoDNodes[i]);
  }

  if (nset >= 3)
  {
    sprintf (NsetNames[1], "x0body");
    sprintf (NsetNames[2], "x1body");
    sprintf (NsetNames[3], "y0body");
    sprintf (NsetNames[4], "y1body");
    sprintf (NsetNames[5], "z0body");
    sprintf (NsetNames[6], "z1body");

    for (i = 1; i <= 6; i++)
      if (nset < 4
	  || (nset == 4
	      && ut_string_inlist (nsetlist, ',', NsetNames[i])) == 1)
	neut_mesh_fprintf_mae_nset (file, NsetNames[i], FoDBNodes[i]);

    sprintf (NsetNames[1], "y0z0body");
    sprintf (NsetNames[2], "y1z0body");
    sprintf (NsetNames[3], "y0z1body");
    sprintf (NsetNames[4], "y1z1body");
    sprintf (NsetNames[5], "x0z0body");
    sprintf (NsetNames[6], "x0z1body");
    sprintf (NsetNames[7], "x1z0body");
    sprintf (NsetNames[8], "x1z1body");
    sprintf (NsetNames[9], "x0y0body");
    sprintf (NsetNames[10], "x1y0body");
    sprintf (NsetNames[11], "x0y1body");
    sprintf (NsetNames[12], "x1y1body");

    for (i = 1; i <= 12; i++)
      if (nset < 4
	  || (nset == 4
	      && ut_string_inlist (nsetlist, ',', NsetNames[i])) == 1)
	neut_mesh_fprintf_mae_nset (file, NsetNames[i], EoDBNodes[i]);
  }

  fprintf (file, "0\n");

  return;
}
