/* This file is ext of the 'neper' program. */
/* Copyright (C) 2003-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"neper_fm.h"

int
neper_fm (int fargc, char **fargv, int argc, char **argv)
{
  char *mname = "-FM";

  /* FILE* debug; */
  int pos, eltid, nodeid;
  struct IN In;
  struct TESS Tess;
  struct GEO Geo;
  struct GEOPARA GeoPara;
  struct NODES Nodes;
  struct MESH TrashMesh0D, TrashMesh1D, Mesh2D, Mesh3D;
  FILE *file;
  char *name, *ext;
  int *FaceList, *EdgeList, *VerList;

  neut_mesh_set_zero (&TrashMesh0D);
  neut_mesh_set_zero (&TrashMesh1D);
  neut_mesh_set_zero (&Mesh2D);
  neut_mesh_set_zero (&Mesh3D);

  /* Printing program header */
  ut_print_moduleheader (mname, fargc, fargv, argc, argv);

  /* Reading input data */
  ut_print_message (0, "Reading input data ...\n");
  InputData_fm (&In, &Tess, &GeoPara, fargc, fargv, argc, argv);
  ut_print_message (0, "Read input data.\n");

  neut_gmsh_rc ("bak");

  /* Creating geometry ----------------------------------------------- */
  if (In.tess2geo)
  {
    /* Creating geometry */
    ut_print_message (0, "Creating geometry ...\n");
    CreateGeo (Tess, &Geo, GeoPara, In);
    ut_print_message (0, "Created geometry.\n");

    /* Meshing vertices (already done if ffalgo!=1) */
    if (In.ffalgo == 1)
      GeoMeshing0D (&Geo, GeoPara);

    if (In.edgedis == 1)	/* Meshing edges */
    {
      ut_print_message (0, "Creating 1-D mesh ...\n");
      GeoMeshing1D (&Geo, GeoPara);
      ut_print_message (0, "Created 1-D mesh.\n");
    }

    if ((GeoPara.maxff > 1e-9 && In.mloop >= 1) || In.meshimprovement2d == 1)
    {
      /* Meshing non-planar faces (recorded as meshes into geo). */
      if (In.meshimprovement2d == 0)
	ut_print_message (0, "Creating 2-D mesh (non-plane faces) ...\n");
      else
	ut_print_message (0, "Creating 2-D mesh ...\n");

      GeoMeshing2D (&Geo, In);

      if (In.meshimprovement2d == 0)
	ut_print_message (0, "Created 2-D mesh (non-plane faces).\n");
      else
	ut_print_message (0, "Created 2-D mesh.\n");
    }

    /* writing of: geo, fod, fff, mod, edgel  */
    WriteGeoStuff (In, Geo, GeoPara, argc, argv, "unknown");
  }

  /* Meshing geometry ---------------------------------------------------- */
  /* or restarting from mesh */
  if (In.mesh)
  {
    if (In.meshimprovement2d == 0)
      ut_print_message (0, "Creating 2-D & 3-D mesh ...\n");
    else
      ut_print_message (0, "Creating 3-D mesh ...\n");

    GeoMeshing3D (In, &Nodes, &Mesh2D, &Mesh3D);	/* writing .msh */

    if (In.meshimprovement2d == 0)
      ut_print_message (0, "Created 2-D & 3-D mesh.\n");
    else
      ut_print_message (0, "Created 3-D mesh.\n");

    file = OpenFile (In.msh, "R");
    ReadMsh (&Nodes, &TrashMesh0D, &TrashMesh1D, &Mesh2D, &Mesh3D, file);
    CloseFile (file, In.msh, "R");
  }
  else if (In.input >= 2)
    ReadMesh (In, &Nodes, &TrashMesh0D, &TrashMesh1D, &Mesh2D, &Mesh3D);

  /* Mesh writing ------------------------------------------------------- */

  /* msh */
  if (In.input < 2 && ut_string_inlist (In.format, ',', "msh") == 0)
    remove (In.msh);

  /* geof */

  if (ut_string_inlist (In.format, ',', "geof"))
  {
    if (In.renumber > 0)
    {
      ut_print_message (0, "Renumbering 3-D mesh...\n");
      eltid = (In.eltfirstid == -1) ? 1 : In.eltfirstid;
      nodeid = (In.nodefirstid == -1) ? 1 : In.nodefirstid;
      MeshRenumber (In.renumber, In.renumberpartitionqty, &Nodes, &Mesh2D, &Mesh3D, eltid, nodeid);
      ut_print_message (0, "Renumbered 3-D mesh.\n");
    }
    
    WriteMesh (In, Geo, Nodes, Mesh2D, Mesh3D, "geof");
  }

  /* mae */
  if (ut_string_inlist (In.format, ',', "mae"))
  {
    if (In.renumber > 0)
    {
      ut_print_message (0, "Renumbering 3-D mesh...\n");
      eltid = (In.eltfirstid == -1) ? 0 : In.eltfirstid;
      nodeid = (In.nodefirstid == -1) ? 0 : In.nodefirstid;
      MeshRenumber (In.renumber, In.renumberpartitionqty, &Nodes, &Mesh2D, &Mesh3D, eltid, nodeid);
      ut_print_message (0, "Renumbered 3-D mesh.\n");
    }

    WriteMesh (In, Geo, Nodes, Mesh2D, Mesh3D, "mae");
  }

  neut_gmsh_rc ("unbak");

  if (In.input < 2 && In.print > 0)
  {
    name = ut_alloc_1d_char (strlen (In.body) + 20);
    ext = ut_alloc_1d_char (strlen (In.body));
    
    PrintGeoStuff (In.print, In.printfaces, Geo, &FaceList, &EdgeList,
		   &VerList);
    pos = 1;
    while (ut_string_part (In.printformat, ',', pos++, ext))
    {
      name = ut_string_addextension (In.body, ext);
      PrintGeoAsImg (Geo, EdgeList, VerList, ext, name);
    }
    ut_free_1d_char (name);
    ut_free_1d_char (ext);
  }

  /* Print mesh statistics ----------------------------------------------- */
  if ((In.mesh || In.input >= 2) && (In.printestat == 1 || In.printnstat == 1))
    MStat (Nodes, Mesh2D, Mesh3D, In);

  /* Removing geo files if needed */
  if (In.input < 1 && ut_string_inlist (In.format, ',', "geo") == 0)
  {
    remove (In.geo);
    remove (In.geob);
    remove (In.fod);
    remove (In.ff);
  }

  return 0;
}
