#include"neut_mesh.h"

/* implemented for geof only */
void
neut_mesh_fscanf (FILE * file, char *format, struct NODES *pNodes,
		  struct MESH *pMesh)
{
  int i, eltnodeqty = 0, dim, order;
  char *string = ut_alloc_1d_char (100);
  char *type = ut_alloc_1d_char (100);

  if (strcmp (format, "geof") != 0)
    abort ();

  /* read header ---------------------------------------------------- */
  fscanf (file, "%s", string);
  if (strcmp (string, "***geometry") != 0)
  {
    ut_print_message (2, "Start word does not match.\n");
    abort ();
  }

  /* read nodes ----------------------------------------------------- */
  fscanf (file, "%s", string);
  if (strcmp (string, "**node") != 0)
  {
    ut_print_message (2, "Start word does not match.\n");
    abort ();
  }

  fscanf (file, "%d%d", &((*pNodes).NodeQty), &((*pMesh).Dimension));

  (*pNodes).NodesNb = ut_alloc_1d_int ((*pNodes).NodeQty + 1);
  (*pNodes).NodesCoo = ut_alloc_2d ((*pNodes).NodeQty + 1, 3);

  for (i = 1; i <= (*pNodes).NodeQty; i++)
  {
    fscanf (file, "%d", &((*pNodes).NodesNb[i]));
    ut_board_1d_fscanf (file, (*pNodes).NodesCoo[i], 3);
  }

  /* calculating NodeNbMax & NodesNbInv ----------------------------- */
  (*pNodes).NodeNbMax =
    ut_board_1d_int_max ((*pNodes).NodesNb + 1, (*pNodes).NodeQty);
  (*pNodes).NodesNbInv = ut_alloc_1d_int ((*pNodes).NodeNbMax + 1);
  for (i = 1; i <= (*pNodes).NodeQty; i++)
    (*pNodes).NodesNbInv[(*pNodes).NodesNb[i]] = i;

  /* reading mesh --------------------------------------------------- */
  fscanf (file, "%s", string);
  if (strcmp (string, "**element") != 0)
  {
    ut_print_message (2, "Start word does not match.\n");
    abort ();
  }
  fscanf (file, "%d", &((*pMesh).EltQty));
  (*pMesh).EltNb = ut_alloc_1d_int ((*pMesh).EltQty + 1);
  (*pMesh).EltNodes = ut_alloc_1d_pint ((*pMesh).EltQty + 1);

  for (i = 1; i <= (*pMesh).EltQty; i++)
  {
    fscanf (file, "%d%s", &((*pMesh).EltNb[i]), string);
    neut_elt_name_prop (format, string, type, &dim, &order);
    if (i == 1)
    {
      (*pMesh).EltOrder = order;
      eltnodeqty = neut_elt_nodeqty (type, dim, order);
    }
    else if ((*pMesh).EltOrder != order || (*pMesh).Dimension != dim)
    {
      ut_print_messagewnc (2, 72,
			   "Mesh dimension and element order must be the same for all elements.");
      abort ();
    }

    (*pMesh).EltNodes[i] = ut_alloc_1d_int (eltnodeqty);
    if ((*pMesh).EltOrder == 1)
      ut_board_1d_int_fscanf (file, (*pMesh).EltNodes[i], eltnodeqty);
    else
    {
      fscanf (file, "%d", &((*pMesh).EltNodes[i][0]));
      fscanf (file, "%d", &((*pMesh).EltNodes[i][1]));
      fscanf (file, "%d", &((*pMesh).EltNodes[i][2]));
      fscanf (file, "%d", &((*pMesh).EltNodes[i][4]));
      fscanf (file, "%d", &((*pMesh).EltNodes[i][5]));
      fscanf (file, "%d", &((*pMesh).EltNodes[i][6]));
      fscanf (file, "%d", &((*pMesh).EltNodes[i][7]));
      fscanf (file, "%d", &((*pMesh).EltNodes[i][9]));
      fscanf (file, "%d", &((*pMesh).EltNodes[i][8]));
      fscanf (file, "%d", &((*pMesh).EltNodes[i][3]));
    }

  }
  
  (*pMesh).EltNbMax =
    ut_board_1d_int_max ((*pMesh).EltNb + 1, (*pMesh).EltQty);
  ut_free_1d_int ((*pMesh).EltNbInv);
  (*pMesh).EltNbInv = ut_alloc_1d_int ((*pMesh).EltNbMax + 1);

  for (i = 1; i <= (*pMesh).EltQty; i++)
    (*pMesh).EltNbInv[(*pMesh).EltNb[i]] = i;

  ut_free_1d_char (string);
  ut_free_1d_char (type);

  return;
}

/* implemented for mae only */
void
neut_maemesh_fscanf (FILE* parms, FILE* mesh, FILE* elsets,
                     struct NODES *pNodes, struct MESH *pMesh)
{
  int i, eltnodeqty = 0;
  char *string = ut_alloc_1d_char (1000);
  /*
  int* tmp;
  int j, eltqty, eltnb, trash, foo;
  int* tmp2;
  */

  fscanf (elsets, "%d", &i); /* for elsets unused */

  /* read parms ----------------------------------------------------- */
  fscanf (parms, "%d%d\n", &(*pMesh).EltQty, &(*pNodes).NodeQty);
  (*pMesh).Dimension = 3;

  /* determining the elt order from the number of words on the first
   * line (elt_id + its nodes) */
  fgets (string, 1000, mesh);
  eltnodeqty = ut_string_nbwords (string) - 1;
  fseek (mesh, 0, 0);

  if (eltnodeqty == 4)
    (*pMesh).EltOrder = 1;
  else if (eltnodeqty == 10)
    (*pMesh).EltOrder = 2;
  else
    abort ();

  (*pMesh).EltNb = ut_alloc_1d_int ((*pMesh).EltQty + 1);
  (*pMesh).EltNodes = ut_alloc_2d_int ((*pMesh).EltQty + 1, eltnodeqty);
  
  /* reading mesh --------------------------------------------------- */
  for (i = 1; i <= (*pMesh).EltQty; i++)
  {
    fscanf (mesh, "%d", &((*pMesh).EltNb[i]));
    if ((*pMesh).EltOrder == 1)
      ut_board_1d_int_fscanf (mesh, (*pMesh).EltNodes[i], eltnodeqty);
    else
    {
      fscanf (mesh, "%d", &((*pMesh).EltNodes[i][0]));
      fscanf (mesh, "%d", &((*pMesh).EltNodes[i][4]));
      fscanf (mesh, "%d", &((*pMesh).EltNodes[i][1]));
      fscanf (mesh, "%d", &((*pMesh).EltNodes[i][5]));
      fscanf (mesh, "%d", &((*pMesh).EltNodes[i][2]));
      fscanf (mesh, "%d", &((*pMesh).EltNodes[i][6]));
      fscanf (mesh, "%d", &((*pMesh).EltNodes[i][7]));
      fscanf (mesh, "%d", &((*pMesh).EltNodes[i][9]));
      fscanf (mesh, "%d", &((*pMesh).EltNodes[i][8]));
      fscanf (mesh, "%d", &((*pMesh).EltNodes[i][3]));
    }
  }
  
  (*pMesh).EltNbMax =
    ut_board_1d_int_max ((*pMesh).EltNb + 1, (*pMesh).EltQty);
  ut_free_1d_int ((*pMesh).EltNbInv);
  (*pMesh).EltNbInv = ut_alloc_1d_int ((*pMesh).EltNbMax + 1);

  ut_file_trash (mesh, 3); /* trashing 1.0 1.0 1.0 */

  /* read nodes ----------------------------------------------------- */
  (*pNodes).NodesNb = ut_alloc_1d_int ((*pNodes).NodeQty + 1);
  (*pNodes).NodesCoo = ut_alloc_2d ((*pNodes).NodeQty + 1, 3);

  for (i = 1; i <= (*pNodes).NodeQty; i++)
  {
    fscanf (mesh, "%d", &((*pNodes).NodesNb[i]));
    ut_board_1d_fscanf (mesh, (*pNodes).NodesCoo[i], 3);
  }

  /* calculating NodeNbMax & NodesNbInv ----------------------------- */
  (*pNodes).NodeNbMax =
    ut_board_1d_int_max ((*pNodes).NodesNb + 1, (*pNodes).NodeQty);
  (*pNodes).NodesNbInv = ut_alloc_1d_int ((*pNodes).NodeNbMax + 1);
  for (i = 1; i <= (*pNodes).NodeQty; i++)
    (*pNodes).NodesNbInv[(*pNodes).NodesNb[i]] = i;

  /* read elsets ---------------------------------------------------- */
  /*
  ut_file_trash (elsets, 2); // trashing grain-input & nb of elements   
  fscanf (elsets, "%d", &((*pMesh).ElsetQty));
  printf ("(*pMesh).ElsetQty = %d\n", (*pMesh).ElsetQty);
  abort ();
  tmp = ut_alloc_1d_int ((*pMesh).EltQty);
  tmp2 = ut_alloc_1d_int ((*pMesh).EltQty);

  for (i = 0; i < (*pMesh).EltQty; i++)
    fscanf (elsets, "%d%d", &(tmp[i]), &trash);

  ut_board_1d_int_memcpy (tmp2, (*pMesh).EltQty, tmp);
  ut_board_1d_int_sortncompress (tmp2, (*pMesh).EltQty, &foo);
  if (foo != (*pMesh).ElsetQty)
  {
    ut_print_messagewnc (2, 72, "Elset quantity does not match.");
    printf ("foo = %d\n", foo);
    printf ("(*pMesh).ElsetQty = %d\n", (*pMesh).ElsetQty);
    abort ();
  }
  (*pMesh).ElsetNb = ut_alloc_1d_int ((*pMesh).ElsetQty + 1);
  ut_board_1d_int_memcpy ((*pMesh).ElsetNb + 1, (*pMesh).ElsetQty, tmp2);
  (*pMesh).ElsetNbMax =
    ut_board_1d_int_max ((*pMesh).ElsetNb + 1, (*pMesh).ElsetQty);

  (*pMesh).ElsetNbInv = ut_alloc_1d_int ((*pMesh).ElsetNbMax + 1);
  for (i = 1; i <= (*pMesh).ElsetQty; i++)
    (*pMesh).ElsetNbInv[(*pMesh).ElsetNb[i]] = i;

  // recording the elements of every elset   
  (*pMesh).Elsets = ut_alloc_1d_pint ((*pMesh).ElsetQty + 1);
  for (i = 1; i <= (*pMesh).ElsetQty; i++)
  {
    eltqty = ut_board_1d_int_nbofthisval (tmp, (*pMesh).EltQty, (*pMesh).ElsetNb[i]);
    (*pMesh).Elsets[i] = ut_alloc_1d_int (eltqty + 1);

    for (j = 0; j < (*pMesh).EltQty; j++)
    {
      if (tmp[j] == (*pMesh).ElsetNb[i])
      {
	eltnb = (*pMesh).EltNb[j + 1];
	(*pMesh).Elsets[i][++(*pMesh).Elsets[i][0]] = eltnb;
      }

      if ((*pMesh).Elsets[i][0] == eltqty)
	break;
    }
  }
  */

  ut_free_1d_char (string);

  return;
}


void
neut_node_elts (struct MESH Mesh, int node, int *elts, int *peltqty)
{
  int i, j;
  int nodeqty = neut_elt_nodeqty ("tri", Mesh.Dimension, Mesh.EltOrder);

  (*peltqty) = 0;
  for (i = 1; i <= Mesh.EltQty; i++)
    for (j = 0; j < nodeqty; j++)
      if (Mesh.EltNodes[i][j] == node)
	elts[(*peltqty)++] = Mesh.EltNb[i];

  ut_board_1d_int_sort (elts, *peltqty);

  return;
}

void
neut_nodes_elts (struct MESH Mesh, int *nodes, int nodeqty,
		 int *elts, int *peltqty)
{
  int i;
  int *tmp = ut_alloc_1d_int (100);
  int *tmp2 = ut_alloc_1d_int (100);
  int tmpqty, tmp2qty;

  neut_node_elts (Mesh, nodes[0], elts, peltqty);
  /*
     printf ("\n\nnode %d elt: ", nodes[0]);
     ut_board_1d_int_fprintf (stdout, elts, *peltqty, "%d");
   */

  for (i = 1; i < nodeqty; i++)
  {
    neut_node_elts (Mesh, nodes[i], tmp, &tmpqty);
    ut_board_1d_int_inter (elts, *peltqty, tmp, tmpqty, tmp2, &tmp2qty);
    ut_board_1d_int_memcpy (elts, tmp2qty, tmp2);
    (*peltqty) = tmp2qty;

    /*
       printf ("node %d elt: ", nodes[i]);
       ut_board_1d_int_fprintf (stdout, tmp, tmpqty, "%d");
       printf ("> elt: ");
       ut_board_1d_int_fprintf (stdout, elts, *peltqty, "%d");
     */
  }

  ut_free_1d_int (tmp);
  ut_free_1d_int (tmp2);

  return;
}

int
neut_nodes_elt (struct MESH Mesh, int *nodes, int nodeqty, int *pelt)
{
  int *tmp = ut_alloc_1d_int (100);
  int tmpqty;

  neut_nodes_elts (Mesh, nodes, nodeqty, tmp, &tmpqty);

  if (tmpqty == 1)
    (*pelt) = tmp[0];
  else
  {
    /*
       printf ("nodes:\n");
       ut_board_1d_int_fprintf (stdout, nodes, nodeqty, "%d");

       printf ("elts: %d\n", tmpqty);
       ut_board_1d_int_fprintf (stdout, tmp, tmpqty, "%d");
     */

    (*pelt) = -1;
  }

  ut_free_1d_int (tmp);

  return ut_num_min (*pelt, 0);
}

int
neut_mesh_eltelset (struct MESH Mesh, int elt, int *pelset)
{
  int i;

  (*pelset) = -1;
  for (i = 1; i <= Mesh.ElsetQty; i++)
    if (ut_board_1d_int_eltpos
	(Mesh.Elsets[i] + 1, Mesh.Elsets[i][0], elt) != -1)
    {
      (*pelset) = Mesh.ElsetNb[i];
      break;
    }

  return ut_num_min (*pelset, 0);
}

int
neut_fod_facefod (int **FoD, int elsetnb, int *pfod)
{
  int i;

  (*pfod) = -1;
  for (i = 1; i <= 6; i++)
    if (ut_board_1d_int_eltpos (FoD[i] + 1, FoD[i][0], elsetnb) != -1)
    {
      (*pfod) = i;
      break;
    }

  return ut_num_min (*pfod, 0);
}

/* can return elt = -1 when a 2d elt has several 3d elts */
int
neut_mesh_elt2delt3d (struct MESH Mesh2D, int elt2d, struct MESH Mesh3D,
		      int *pelt3d)
{
  int i;
  int eltnbinv = Mesh2D.EltNbInv[elt2d];
  int nodeqty = neut_elt_nodeqty ("tri", 2, Mesh2D.EltOrder);
  int status;

  status =
    neut_nodes_elt (Mesh3D, Mesh2D.EltNodes[eltnbinv], nodeqty, pelt3d);
  if (status < 0)
  {
    printf ("neut_nodes_elt returned %d.\n", status);

    for (i = 1; i <= Mesh2D.EltQty; i++)
    {
      printf ("EltNb = %d EltNbInv = %d\n", Mesh2D.EltNb[i],
	      Mesh2D.EltNbInv[i]);
      printf ("EltNodes = ");
      ut_board_1d_int_fprintf (stdout, Mesh2D.EltNodes[i], 3, "%d");
    }
  }

  return ut_num_min (*pelt3d, 0);
}

void
neut_mesh_elt_reversenodes (struct MESH Mesh, int eltnb)
{
  int eltnbinv = Mesh.EltNbInv[eltnb];
  int eltnodeqty = neut_elt_nodeqty ("tri", Mesh.Dimension, Mesh.EltOrder);

  ut_board_1d_int_reverseelts (Mesh.EltNodes[eltnbinv], eltnodeqty);

  return;
}

void
neut_mesh_eltnormal (struct MESH Mesh, struct NODES Nodes, int eltnb,
		     double *n)
{
  int i;
  int eltnbinv = Mesh.EltNbInv[eltnb];
  int *nodenbinv = ut_alloc_1d_int (3);

  for (i = 0; i < 3; i++)
    nodenbinv[i] = Nodes.NodesNbInv[Mesh.EltNodes[eltnbinv][i]];

  ut_space_trianglenormal (Nodes.NodesCoo[nodenbinv[0]],
			   Nodes.NodesCoo[nodenbinv[1]],
			   Nodes.NodesCoo[nodenbinv[2]], n);

  ut_free_1d_int (nodenbinv);

  return;
}

void
neut_mesh_set_zero (struct MESH *pMesh)
{
  (*pMesh).Dimension = 0;
  (*pMesh).EltOrder = 0;
  (*pMesh).EltQty = 0;
  (*pMesh).EltNbMax = 0;
  (*pMesh).ElsetQty = 0;
  (*pMesh).ElsetNbMax = 0;
  (*pMesh).EltQty4Alloc = 0;

  (*pMesh).EltNb = NULL;
  (*pMesh).EltNbInv = NULL;
  (*pMesh).EltNodes = NULL;
  (*pMesh).ElsetNb = NULL;
  (*pMesh).ElsetNbInv = NULL;
  (*pMesh).Elsets = NULL;

  return;
}

int
neut_mesh_elt_elset (struct MESH Mesh, int eltnb)
{
  int i;
  int elset = -1;

  for (i = 1; i <= Mesh.ElsetQty; i++)
    if (ut_board_1d_int_eltpos (Mesh.Elsets[i] + 1, 
	                        Mesh.Elsets[i][0], eltnb) != -1)
    {
      elset = Mesh.ElsetNb[i];
      break;
    }

  return elset;
}

void
neut_mesh_elsetnodes (struct MESH Mesh, int id, int **pnodes, int *pqty)
{
  int i, elt, eltinv, qty, eltnodeqty;
  int *tmp;
  int idinv = Mesh.ElsetNbInv[id];

  eltnodeqty = neut_elt_nodeqty ("tri", Mesh.Dimension, Mesh.EltOrder);

  tmp = ut_alloc_1d_int (Mesh.Elsets[idinv][0] * eltnodeqty);

  qty = 0;
  for (i = 1; i <= Mesh.Elsets[idinv][0]; i++)
  {
    elt = Mesh.Elsets[idinv][i];
    eltinv = Mesh.EltNbInv[elt];

    ut_board_1d_int_memcpy (tmp + qty, eltnodeqty, Mesh.EltNodes[eltinv]);
    qty += eltnodeqty;
  }

  ut_board_1d_int_sortncompress (tmp, qty, &qty);

  (*pqty) = qty;
  (*pnodes) = ut_alloc_1d_int (qty);
  ut_board_1d_int_memcpy (*pnodes, qty, tmp);

  ut_free_1d_int (tmp);

  return;
}


/* crude computation of elset centre: averaging of its node coo */
/* we should use weighted element centres */
void
neut_mesh_elsetcentre (struct NODES Nodes, struct MESH Mesh, int id, double *coo)
{
  int i, j, qty, elsetinv, nodeinv;
  int* nodenbs;

  elsetinv = Mesh.ElsetNbInv[id];

  neut_mesh_elsetnodes (Mesh, elsetinv, &nodenbs, &qty);

  ut_board_1d_set (coo, 3, 0);
  for (i = 0; i < qty; i++)
  {
    nodeinv = Nodes.NodesNbInv[nodenbs[i]];

    for (j = 0; j < 3; j++)
      coo[j] += Nodes.NodesCoo[nodeinv][j];
  }
  for (j = 0; j < 3; j++)
    coo[j] /= qty;

  ut_free_1d_int (nodenbs);

  return;
}
