#include "neut_mesh_renumber.h"

void
neut_nodes_renumber_continuous (struct NODES *pNodes, int beg, int *nodes_old_new)
{
  int corr;

  if (beg < 0)
  {
    ut_print_message (2, "Node numbers cannot be negative.\n");
    abort ();
  }

  corr = beg - 1;

  ut_board_1d_int_memcpy (nodes_old_new, (*pNodes).NodeNbMax + 1,
			  (*pNodes).NodesNbInv);

  ut_board_1d_int_addval (nodes_old_new, (*pNodes).NodeNbMax + 1, corr,
 			  nodes_old_new);

  neut_nodes_renumber (pNodes, nodes_old_new);
  
  neut_nodes_sortbynumber (pNodes);

  return;
}
  

void
neut_mesh_renumber_continuous (struct MESH *pMesh, int beg, int *nodes_old_new,
		    int *elts_old_new)
{
  int i, corr;

  if ((*pMesh).EltQty == 0)
    return;

  if (beg < 0)
  {
    ut_print_message (2, "Element numbers cannot be negative.\n");
    abort ();
  }

  /* building elts_old_new */
  ut_board_1d_int_memcpy (elts_old_new, (*pMesh).EltNbMax + 1,
			  (*pMesh).EltNbInv);
  corr = beg - 1;
  if (corr != 0)
    for (i = 1; i <= (*pMesh).EltNbMax; i++)
      elts_old_new[i] += corr;
      
  neut_mesh_renumber (pMesh, nodes_old_new, elts_old_new);
  
  neut_mesh_sortbynumber (pMesh);

  return;
}


/* renumber_continuous must have been called before */
void
neut_mesh_renumber_crude (struct NODES *pNodes, struct MESH* pMesh2D, struct MESH *pMesh)
{
  int i, size;
  double** GrainCentre = ut_alloc_2d ((*pMesh).ElsetQty + 1, 3);
  int* sequence = ut_alloc_1d_int ((*pMesh).ElsetQty + 1);
  double** boundingbox = ut_alloc_2d (3, 2);

  neut_nodes_boundingbox (*pNodes, boundingbox);

  for (i = 1; i <= (*pMesh).ElsetQty; i++)
    neut_mesh_elsetcentre (*pNodes, *pMesh, (*pMesh).ElsetNb[i], GrainCentre[i]);

  size = ut_num_d2ri (pow ((*pMesh).ElsetQty, 0.33333));
  ut_space_arrangepoints (GrainCentre + 1, (*pMesh).ElsetQty, boundingbox, size, sequence);
  ut_board_1d_int_addval (sequence + 1, sequence[0], 1, sequence + 1);

  neut_mesh_renumber_bypoly (sequence, pNodes, pMesh2D, pMesh);

  ut_free_1d_int (sequence);
  ut_free_2d (GrainCentre, (*pMesh).ElsetQty + 1);
  ut_free_2d (boundingbox, 3);

  return;
}

