#include "neut_mesh_renumber.h"

void
neut_nodes_renumber (struct NODES *pNodes, int *nodes_old_new)
{
  int i;

  for (i = 1; i <= (*pNodes).NodeQty; i++)
    (*pNodes).NodesNb[i] = nodes_old_new[(*pNodes).NodesNb[i]];

  /* correcting node nb inv; realloc to redefine size (minimum) */
  (*pNodes).NodeNbMax =
    ut_board_1d_int_max ((*pNodes).NodesNb + 1, (*pNodes).NodeQty);
  ut_free_1d_int ((*pNodes).NodesNbInv);
  (*pNodes).NodesNbInv = ut_alloc_1d_int ((*pNodes).NodeNbMax + 1);

  for (i = 1; i <= (*pNodes).NodeQty; i++)
    (*pNodes).NodesNbInv[(*pNodes).NodesNb[i]] = i;

  return;
}

void
neut_nodes_sortbynumber (struct NODES *pNodes)
{
  int i;
  int min = ut_board_1d_int_min ((*pNodes).NodesNb + 1, (*pNodes).NodeQty);

  /* Nodes needs to be continuously numbered */
  if ((*pNodes).NodeNbMax != min + (*pNodes).NodeQty - 1)
  {
    printf ("bad Nodes format for neut_nodes_sortbynumber\n");
    abort ();
  }

  for (i = 1; i <= (*pNodes).NodeQty; i++)
    neut_nodes_switch (pNodes, min + i - 1, (*pNodes).NodesNb[i]);

  return;
}

void
neut_mesh_sortbynumber (struct MESH *pMesh)
{
  int i;
  int min = ut_board_1d_int_min ((*pMesh).EltNb + 1, (*pMesh).EltQty);

  /* Mesh needs to be continuously numbered */
  if ((*pMesh).EltNbMax != min + (*pMesh).EltQty - 1)
  {
    printf ("bad Mesh format for neut_mesh_sortbynumber\n");
    abort ();
  }

  for (i = 1; i <= (*pMesh).EltQty; i++)
    neut_mesh_switch (pMesh, min + i - 1, (*pMesh).EltNb[i]);

  return;
}

void
neut_mesh_renumber (struct MESH *pMesh, int *nodes_old_new, int *elts_old_new)
{
  int i, j, eltnodeqty;

  /* correcting elt nodes */
  eltnodeqty =
    neut_elt_nodeqty ("tri", (*pMesh).Dimension, (*pMesh).EltOrder);

  for (i = 1; i <= (*pMesh).EltQty; i++)
    for (j = 0; j < eltnodeqty; j++)
      (*pMesh).EltNodes[i][j] = nodes_old_new[(*pMesh).EltNodes[i][j]];

  /* correcting elt numbers */

  for (i = 1; i <= (*pMesh).EltQty; i++)
    (*pMesh).EltNb[i] = elts_old_new[(*pMesh).EltNb[i]];

  /* correcting elt nb inv; realloc to redefine size (minimum) */
  (*pMesh).EltNbMax =
    ut_board_1d_int_max ((*pMesh).EltNb + 1, (*pMesh).EltQty);
  ut_free_1d_int ((*pMesh).EltNbInv);
  (*pMesh).EltNbInv = ut_alloc_1d_int ((*pMesh).EltNbMax + 1);

  for (i = 1; i <= (*pMesh).EltQty; i++)
    (*pMesh).EltNbInv[(*pMesh).EltNb[i]] = i;

  /* correcting elsets */
  for (i = 1; i <= (*pMesh).ElsetQty; i++)
    for (j = 1; j <= (*pMesh).Elsets[i][0]; j++)
      (*pMesh).Elsets[i][j] = elts_old_new[(*pMesh).Elsets[i][j]];

  return;
}



  
void
neut_mesh_renumber_bypoly (int* sequence, struct NODES* pNodes,
                           struct MESH* pMesh2D, struct MESH* pMesh)
{
  int i, j, k, neweltid, newnodeid, elsetidinv, eltid, eltidinv, nodeid;
  int eltnodeqty;
  int* elts_old_new;
  int* nodes_old_new;

  elts_old_new = ut_alloc_1d_int ((*pMesh).EltNbMax + 1);
  nodes_old_new = ut_alloc_1d_int ((*pNodes).NodeNbMax + 1);
  ut_board_1d_int_set (nodes_old_new, (*pNodes).NodeNbMax + 1, -1);

  eltnodeqty =
    neut_elt_nodeqty ("tri", (*pMesh).Dimension, (*pMesh).EltOrder);

  /* renumber by polyhedron */

  neweltid = (*pMesh).EltNb[1] - 1;
  newnodeid = (*pNodes).NodesNb[1] - 1;

  /* for every poly */
  for (i = 1; i <= sequence[0]; i++)
  {
    elsetidinv = sequence[i];

    /* for every elt */
    for (j = 1; j <= (*pMesh).Elsets[elsetidinv][0]; j++)
    {
      eltid    = (*pMesh).Elsets[elsetidinv][j];
      eltidinv = (*pMesh).EltNbInv[eltid];

      elts_old_new [eltid] = ++neweltid;

      for (k = 0; k < eltnodeqty; k++)
      {
	nodeid = (*pMesh).EltNodes[eltidinv][k];

	if (nodes_old_new[nodeid] == -1)
	  nodes_old_new [nodeid] = ++newnodeid;
      }
    }
  }

  if (neweltid != (*pMesh).EltQty || newnodeid != (*pNodes).NodeQty)
  {
    printf ("bad elt and / or node renumbering\n");
    printf ("neweltid = %d != (*pMesh).EltQty = %d\n", neweltid, (*pMesh).EltQty);
    printf ("newnodeid = %d != (*pNodes).NodeQty = %d\n", newnodeid, (*pNodes).NodeQty);

    abort ();
  }

  neut_nodes_renumber (pNodes, nodes_old_new);
  neut_mesh_renumber (pMesh, nodes_old_new, elts_old_new);
  ut_free_1d_int (elts_old_new);
  
  elts_old_new = ut_alloc_1d_int ((*pMesh2D).EltNbMax + 1);
  neut_mesh_renumber_continuous (pMesh2D, (*pMesh2D).EltNb[1], nodes_old_new, elts_old_new);

  neut_nodes_sortbynumber (pNodes);
  neut_mesh_sortbynumber (pMesh);
  
  ut_free_1d_int (elts_old_new);
  ut_free_1d_int (nodes_old_new);

  return;
}
