#include"neut_nodes.h"

void
neut_nodes_scale (struct NODES *pNodes, double scalex, double scaley,
		  double scalez)
{
  int i;

  for (i = 1; i <= (*pNodes).NodeQty; i++)
  {
    (*pNodes).NodesCoo[i][0] *= scalex;
    (*pNodes).NodesCoo[i][1] *= scaley;
    (*pNodes).NodesCoo[i][2] *= scalez;
  }

  return;
}

void
neut_nodes_shift (struct NODES *pNodes, double shiftx, double shifty,
		  double shiftz)
{
  int i;

  for (i = 1; i <= (*pNodes).NodeQty; i++)
  {
    (*pNodes).NodesCoo[i][0] += shiftx;
    (*pNodes).NodesCoo[i][1] += shifty;
    (*pNodes).NodesCoo[i][2] += shiftz;
  }

  return;
}

void
neut_nodes_domain (struct NODES Nodes, double* domain)
{
  int i;

  for (i = 0; i < 3; i++)
  {
    domain[2 * i]     =  1e32;
    domain[2 * i + 1] = -1e32;
  }

  for (i = 1; i <= Nodes.NodeQty; i++)
  {
    domain[0] = ut_num_min (domain[0], Nodes.NodesCoo[i][0]);
    domain[1] = ut_num_max (domain[1], Nodes.NodesCoo[i][0]);
    domain[2] = ut_num_min (domain[2], Nodes.NodesCoo[i][1]);
    domain[3] = ut_num_max (domain[3], Nodes.NodesCoo[i][1]);
    domain[4] = ut_num_min (domain[4], Nodes.NodesCoo[i][2]);
    domain[5] = ut_num_max (domain[5], Nodes.NodesCoo[i][2]);
  }

  return;
}

void
neut_nodes_boundingbox (struct NODES Nodes, double** size)
{
  /* assuming a cubic box aligned with the X, Y and Z axes */
  int i, j;

  size[0][0] = size[1][0] = size[2][0] = 1e32;
  size[0][1] = size[1][1] = size[2][1] = -1e32;

  for (i = 0; i < Nodes.NodeQty; i++)
  {
    for (j = 0; j < 3; j++)
    {
      size[j][0] = ut_num_min (size[j][0], Nodes.NodesCoo[i][j]);
      size[j][1] = ut_num_max (size[j][1], Nodes.NodesCoo[i][j]);
    }
  }

  return;
}



void
neut_nodes_fodnodes (struct NODES Nodes, int ***pFoDNodes)
{
  int face, i, coo;
  int* tmp = ut_alloc_1d_int (Nodes.NodeQty + 1);
  double* domain = ut_alloc_1d (6);
  double eps = 1e-12;

  neut_nodes_domain (Nodes, domain);

  for (i = 0; i < 3; i++)
  {
    domain[2 * i] += eps;
    domain[2 * i + 1] -= eps;
  }

  (*pFoDNodes) = ut_alloc_1d_pint (7);

  for (face = 1; face <= 6; face++)
  {
    if (face == 1 || face == 2)
      coo = 0;
    else if (face == 3 || face == 4)
      coo = 1;
    else
      coo = 2;

    tmp[0] = 0;
    if (face % 2 == 1)
    {
      for (i = 1; i <= Nodes.NodeQty; i++)
	if (Nodes.NodesCoo[i][coo] < domain[face - 1])
	  tmp[++tmp[0]] = i;
    }
    else
    {
      for (i = 1; i <= Nodes.NodeQty; i++)
	if (Nodes.NodesCoo[i][coo] > domain [face - 1])
	  tmp[++tmp[0]] = i;
    }

    (*pFoDNodes)[face] = ut_alloc_1d_int (tmp[0] + 1);
    ut_board_1d_int_memcpy ((*pFoDNodes)[face], tmp[0] + 1, tmp);
  }
  
  return;
}


